<?php 
/**
* This file is part of the jVitals UserSync distribution. 
* Detailed copyright and licensing information can be found
* in the gpl-3.0.txt file which should be included in the distribution.
* 
* @version		$Id: 1.0.01 2010-10-08 rammstein4o $
* @copyright  2007 - 2010 jVitals
* @license    GPLv3 Open Source
* @link       http://jvitals.com
* @since      File available since initial release
*/
 
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport('joomla.plugin.plugin');

class plgUserJvitals_usersync extends JPlugin {

	/**
	 * Constructor
	 *
	 * For php4 compatability we must not use the __constructor as a constructor for plugins
	 * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
	 * This causes problems with cross-referencing necessary for the observer design pattern.
	 *
	 * @param object $subject The object to observe
	 * @param 	array  $config  An array that holds the plugin configuration
	 * @since 1.5
	 */
	function plgUserJvitals_usersync(&$subject, $config) {
		parent::__construct($subject, $config);
	}
	
    /**
     * Verifies jSocialSuite is installed 
     * @return boolean
     */
    private function _jssInstalled() {
        jimport('joomla.filesystem.file');
        if (JFile::exists(JPATH_SITE . DS . 'components' . DS . 'com_jsocialsuite' . DS . 'jsocialsuite.php')) {
            return true;
        }
        return false;
    }
	
    /**
     * Verifies Agora is installed 
     * @return boolean
     */
    private function _agoraInstalled() {
        jimport('joomla.filesystem.file');
        if (JFile::exists(JPATH_SITE . DS . 'components' . DS . 'com_agora' . DS . 'agora.php')) {
            return true;
        }
        return false;
    }
	
    /**
     * Verifies Extendedreg is installed 
     * @return boolean
     */
    private function _extregInstalled() {
        jimport('joomla.filesystem.file');
        if (JFile::exists(JPATH_SITE . DS . 'components' . DS . 'com_extendedreg' . DS . 'extendedreg.php')) {
            return true;
        }
        return false;
    }

	/**
	 * Method is called after user data is stored in the database
	 *
	 * @param 	array		holds the new user data
	 * @param 	boolean		true if a new user is stored
	 * @param	boolean		true if user was succesfully stored in the database
	 * @param	string		message
	 */
	function onAfterStoreUser($user, $isnew, $success, $msg) {
		global $mainframe;
		
		if ($isnew && $success) {
			if ($this->_jssInstalled()) $this->synchronizeJssUser($user['id']);
			if ($this->_extregInstalled()) $this->synchronizeExtregUser($user['id']);
		}
		
		if ($success) {
			if ($this->_agoraInstalled()) $this->synchronizeAgoraUser($user, (boolean)!$isnew);
		}
	}
	
	/**
	 * Method is called after user data is deleted from the database
	 *
	 * @param 	array		holds the user data
	 * @param	boolean		true if user was succesfully stored in the database
	 * @param	string		message
	 */
	function onAfterDeleteUser($user, $success, $msg) {
		global $mainframe;

		if ($success) {
			if ($this->_jssInstalled()) $this->synchronizeJssUser($user['id'], true);
			if ($this->_extregInstalled()) $this->synchronizeExtregUser($user['id'], true);
			if ($this->_agoraInstalled()) $this->synchronizeAgoraUser($user, false, true);
		}
	}

	/**
     * Synchronizes a Joomla User with the jSocialSuite Users DB
     * 
     * @param 	array 	holds the user data
	 * @return	boolean	True on success
     */
	function synchronizeJssUser($userid, $delete = false) {
		if (!(int)$userid) return false;
		@define('JSS_APPS_INCPATH', JPATH_SITE . DS . 'components' . DS . 'com_jsocialsuite' . DS . 'apps' . DS);
		require_once (JPATH_SITE . DS . 'components' . DS . 'com_jsocialsuite' . DS . 'libraries' . DS . 'fns_support.php');
		require_once (JPATH_SITE . DS . 'components' . DS . 'com_jsocialsuite' . DS . 'libraries' . DS . 'jssfactory.php');
		require_once (JPATH_SITE . DS . 'components' . DS . 'com_jsocialsuite' . DS . 'models' . DS . 'user.php');
		require_once (JPATH_SITE . DS . 'components' . DS . 'com_jsocialsuite' . DS . 'helpers' . DS . 'params.php');
		
		$dbo = JFactory::getDBO();
		if ($delete) {
			$dbo->setQuery("DELETE FROM #__jss_users WHERE " . $dbo->nameQuote('user_id') . " = " . (int)$userid);
			$dbo->query();
		} else {
			$model = new jssModelUser();
			$model->createIfNotExists((int)$userid);
		}
		return true;
	}
	
	/**
     * Synchronizes a Joomla User with the Extendedreg Users DB
     * 
     * @param 	array 	holds the user data
	 * @return	boolean	True on success
     */
	function synchronizeExtregUser($userid, $delete = false) {
		if (!(int)$userid) return false;
		$dbo = JFactory::getDBO();
		if ($delete) {
			$dbo->setQuery("DELETE FROM #__extendedreg_users WHERE " . $dbo->nameQuote('user_id') . " = " . (int)$userid);
			$dbo->query();
		} else {
			$dbo->setQuery("INSERT INTO #__extendedreg_users (" . $dbo->nameQuote('user_id') . ") VALUES (" . (int)$userid . ")");
			$dbo->query();
		}
		return true;
	}

	/**
     * Synchronizes a Joomla User with the Agora Users DB
     * 
     * @param 	array 	holds the user data
	 * @return	boolean	True on success
     */
	function synchronizeAgoraUser($user, $update, $delete = false) {
		if (!(int)$userid) return false;
		$dbo = JFactory::getDBO();
		if ($delete) {
			$dbo->setQuery("DELETE FROM #__agora_users WHERE " . $dbo->nameQuote('jos_id') . " = " . (int)$user['id']);
			$dbo->query();
		} else {
			if ($update) {
				$sql = "UPDATE #__agora_users SET 
					" . $dbo->nameQuote('username') . " = " . $dbo->Quote($user['username']) . ",
					" . $dbo->nameQuote('email') . " = " . $dbo->Quote($user['email']) . "
				WHERE " . $dbo->nameQuote('jos_id') . " = " . (int)$user['id'];
			} else {
				$sql = "INSERT INTO #__agora_users (" . $dbo->nameQuote('jos_id') . ", " . $dbo->nameQuote('username') . ", " . $dbo->nameQuote('email') . ") 
				VALUES (" . (int)$user['id'] . ", " . $dbo->Quote($user['username']) . ", " . $dbo->Quote($user['email']) . ")";
			}
			
			$dbo->setQuery($sql);
			$dbo->query();
		}
		return true;
	}
}
