<?php
/**
 * This file is part of the Agora distribution. 
 * Detailed copyright and licensing information can be found
 * in the gpl-3.0.txt file which should be included in the distribution.
 * 
 * @version		$Id: 3.0.14 Olympus 2010-09-28 rammstein4o $
 * @copyright  2007 - 2010 jVitals
 * @license   GPLv3 Open Source
 * @link       http://jvitals.com
 * @since      File available since initial release
 */
	class SubscriptionModel extends Model
	{
		function __construct()
		{
			parent::__construct('subscriptions');
		}

		function isSubscribed($user_id, $topic_id)
		{
			$this->db->bind('user_id',$user_id,'integer');
			$this->db->bind('topic_id',$topic_id,'integer');

			$this->db->setQuery('SELECT COUNT(*) FROM ##__subscriptions'.
							' WHERE user_id=:user_id'.
							' AND topic_id=:topic_id');
			return intval($this->db->loadResult());
		}

		function delete($user_id, $topic_id)
		{
			$this->db->bind('user_id',$user_id,'integer');
			$this->db->bind('topic_id',$topic_id,'integer');

			$this->db->setQuery('DELETE FROM ##__subscriptions'.
							' WHERE user_id=:user_id'.
							' AND topic_id=:topic_id');
			$this->db->query();
		}
		
		function delete_forum($user_id, $forum_id)
		{
			$this->db->bind('user_id',$user_id,'integer');
			$this->db->bind('forum_id',$forum_id,'integer');

			$this->db->setQuery('DELETE FROM ##__subscriptions'.
							' WHERE user_id=:user_id'.
							' AND forum_id=:forum_id');
			$this->db->query();
		}


		function loadSubscribers($topic_id, $forum_id)
		{
			$this->db->bind('topic_id',$topic_id,'integer');
			$this->db->bind('forum_id',$forum_id,'integer');

			$this->db->setQuery('SELECT DISTINCT user.id, user.email, user.username, user.disp_posts, j_u.name, s.emailed, s.topic_id, s.forum_id'.
							' FROM ##__users AS user'.
							' INNER JOIN ##__subscriptions AS s'.
							'  ON s.user_id = user.id'.
							' LEFT JOIN #__users AS j_u'.
							'  ON j_u.id = user.jos_id'.
							' WHERE (s.topic_id=:topic_id OR s.forum_id=:forum_id)');
			return $this->db->loadAssocList();
		}
		
		function updateSubscriberEmailed($user_id, $topic_id, $forum_id, $emailed)
		{
            $this->db->setQuery("UPDATE ##__subscriptions SET emailed=$emailed
                            	WHERE user_id = $user_id AND topic_id = $topic_id AND forum_id=$forum_id");
            return $this->db->query();
		}
		
		function addAllMemRolToAllThereForums($role)
		{
				$this->db->setQuery('SELECT group_forum.forum_id, user_group.user_id'.
								' FROM ##__user_group AS user_group'.
								' INNER JOIN ##__group_forum AS group_forum'.
								'  ON user_group.group_id = group_forum.group_id'.
								'  AND user_group.role_id = group_forum.role_id'.
								' INNER JOIN ##__permissions AS permissions'.
								'  ON permissions.id = group_forum.access_id'.
				                ' INNER JOIN ##__roles as r ON user_group.role_id=r.id'.
								' WHERE r.name=\''.$role.'\'' );
				$result = $this->db->loadAssocList();
				foreach ($result as $res)
				{
				    $this->add(array('user_id'=>$res['user_id'],'forum_id'=>$res['forum_id']));
				}
		}
		
		function removeAllMemRolToAllThereForums($role)
		{
				$this->db->setQuery('SELECT group_forum.forum_id, user_group.user_id'.
								' FROM ##__user_group AS user_group'.
								' INNER JOIN ##__group_forum AS group_forum'.
								'  ON user_group.group_id = group_forum.group_id'.
								'  AND user_group.role_id = group_forum.role_id'.
								' INNER JOIN ##__permissions AS permissions'.
								'  ON permissions.id = group_forum.access_id'.
				                ' INNER JOIN ##__roles as r ON user_group.role_id=r.id'.
								' WHERE r.name=\''.$role.'\'' );
				$result = $this->db->loadAssocList();
				foreach ($result as $res)
				{
				    $this->db->setQuery("DELETE FROM ##__subscriptions WHERE user_id=".$res['user_id']." AND forum_id=".$res['forum_id']);
		            $this->db->query();
				}
		}
		
		function addAllMemRolToForums($role, $forums)
		{
			$this->db->setQuery('SELECT u.id FROM ##__users as u INNER JOIN ##__user_group as u_g ON u.id=u_g.user_id INNER JOIN ##__roles as r ON u_g.role_id=r.id WHERE u_g.group_id=1 AND r.name=\''.$role.'\' ');
			$admins = $this->db->loadResultArray();
		    foreach ($admins as $u_id)
		        foreach ($forums as $forum)
		        {
		            $this->add(array('user_id'=>$u_id,'forum_id'=>$forum['id']));
		        }
		}
		
		function removeAllMemRolFromForums($role, $forums)
		{
			$this->db->setQuery('SELECT u.id FROM ##__users as u INNER JOIN ##__user_group as u_g ON u.id=u_g.user_id INNER JOIN ##__roles as r ON u_g.role_id=r.id WHERE u_g.group_id=1 AND r.name=\''.$role.'\' ');
			$admins = $this->db->loadResultArray();
		    foreach ($admins as $u_id)
		        foreach ($forums as $forum)
		        {
		            $this->db->setQuery("DELETE FROM ##__subscriptions WHERE user_id=$u_id AND forum_id=".$forum['id']);
		            $this->db->query();
		        }
		}
		
		function getSubscribedTopics($user_id, $per_page = null, $page = null)
		{
			$limit = $this->getLimit($per_page, $page);
			
			$this->db->bind('user_id',$user_id,'integer');

			$this->db->setQuery('SELECT topics.id, topics.subject, topics.poster, topics.num_replies'.
							' FROM ##__topics AS topics'.
							' INNER JOIN ##__subscriptions AS s'.
							' ON s.topic_id = topics.id'.
							' WHERE s.user_id=:user_id ' . $limit );
			return $this->db->loadAssocList();
		}
		
		function countSubscribedTopics($user_id)
		{
			$this->db->bind('user_id',$user_id,'integer');

			$this->db->setQuery('SELECT COUNT(*)'.
							' FROM ##__topics AS topics'.
							' INNER JOIN ##__subscriptions AS s'.
							' ON s.topic_id = topics.id'.
							' WHERE s.user_id=:user_id ');
			return $this->db->loadResult();
		}
		
		function getSubscribedForums($user_id)
		{
			$this->db->bind('user_id',$user_id,'integer');

			$this->db->setQuery('SELECT forums.id, forums.forum_name, forums.num_posts, forums.num_topics,'.
							' forums.forum_desc FROM ##__forums AS forums'.
							' INNER JOIN ##__subscriptions AS s'.
							' ON s.forum_id = forums.id'.
							' WHERE s.user_id=:user_id ');
			return $this->db->loadAssocList();
		}
		
		function countSubscribedForums($user_id)
		{
			$this->db->bind('user_id',$user_id,'integer');

			$this->db->setQuery('SELECT COUNT(*)'.
							' FROM ##__forums AS forums'.
							' INNER JOIN ##__subscriptions AS s'.
							' ON s.forum_id = forums.id'.
							' WHERE s.user_id=:user_id ');
			return $this->db->loadResult();
		}

	}
?>
